using System;
using System.ComponentModel;
using System.Reactive.Linq;
using System.Text.RegularExpressions;
using DynamicData;
using DynamicData.Binding;
using Splat;
using StarsAssistant.Model;
using StarsAssistant.ViewModels;

namespace StarsAssistant.Services;

public class FleetManager
{
    protected Services.Game Game = Locator.Current.GetService<Services.Game>()!;

    /// <summary>
    /// SourceCache for DynamicData views
    /// </summary> 
    private SourceCache<Fleet, int> _fleets = new(f => f.Id);

    public FleetManager() {}

    /// <summary>
    /// Load the fleet records from the database and push them into our source cache.
    /// If the data has been freshly imported, call PostProcessImportedData first.
    /// </summary>
    public void InitFromDatabase()
    {
        using var db = Locator.Current.GetService<StarsDatabase>()!;
        var allFleets = db.Fleet.ToList();
        _fleets.AddOrUpdate(allFleets);
    }

    /// <summary>
    /// Helper to fill up missing data from the original import. Tries to deduce
    /// missing properties using heuristics.
    /// </summary> 
    public static void PostProcessImportedData()
    {
        using var db = Locator.Current.GetService<StarsDatabase>()!;

        // Check for all cases where we're targeting another fleet instead of
        // a planet. Update the DB accordingly.
        var playerFleets = from flt in db.Fleet
            where flt.OwnerFileId == Game.Player.PlayerFileId
            select flt;

        Regex shipPattern = new ($@"^{Regex.Escape(Game.Player.Name)} (?<ShipType>.+) #\d+$");

        foreach (Fleet flt in playerFleets)
        {
            if (flt.Destination != "-- ")
            {
                var trueDest = db.Fleet
                    .FirstOrDefault(f => f.FleetName == $"{Game.Player.Name} {flt.Destination}"
                        && f.Planet != String.Empty);
                flt.TrueDestination = trueDest?.Planet ?? flt.Destination;
            }
            else
            {
                flt.TrueDestination = "-- ";
            }
            
            Match m = shipPattern.Match(flt.FleetName);
            if (m.Success)
                flt.ShipTypeGuess = m.Groups[1].Value;

            db.Update(flt);            
        }

        db.SaveChanges();
    }

}
